@@ -1,6 +1,6 @@
-// Specific definitions for Solaris 7,8  -*- C++ -*-
+// Specific definitions for Solaris 2.6  -*- C++ -*-
 
-// Copyright (C) 2000, 2002 Free Software Foundation, Inc.
+// Copyright (C) 2000, 2002, 2005 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -15,7 +15,7 @@
 
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
 // As a special exception, you may use this file as part of a free software
@@ -33,11 +33,4 @@
 // System-specific #define, typedefs, corrections, etc, go here.  This
 // file will come before all others.
 
-// These are typedefs which libio assumes are already in place (because
-// they really are, under Linux).
-#define __off_t     off_t
-#define __off64_t   off64_t
-#define __ssize_t   ssize_t
-
 #endif
-
