@@ -1,6 +1,6 @@
 // MT-optimized allocator -*- C++ -*-
 
-// Copyright (C) 2003, 2004, 2005 Free Software Foundation, Inc.
+// Copyright (C) 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -37,11 +37,14 @@
 #include <new>
 #include <cstdlib>
 #include <bits/functexcept.h>
-#include <bits/gthr.h>
-#include <bits/atomicity.h>
+#include <ext/atomicity.h>
+#include <bits/stl_move.h>
+
+_GLIBCXX_BEGIN_NAMESPACE(__gnu_cxx)
+
+  using std::size_t;
+  using std::ptrdiff_t;
 
-namespace __gnu_cxx
-{
   typedef void (*__destroy_handler)(void*);
 
   /// @brief  Base class for pool object.
@@ -83,7 +86,7 @@ namespace __gnu_cxx
       // In order to avoid fragmenting and minimize the number of
       // new() calls we always request new memory using this
       // value. Based on previous discussions on the libstdc++
-      // mailing list we have choosen the value below.
+      // mailing list we have chosen the value below.
       // See http://gcc.gnu.org/ml/libstdc++/2001-07/msg00077.html
       // NB: At least one order of magnitude > _M_max_bytes. 
       size_t	_M_chunk_size;
@@ -112,7 +115,7 @@ namespace __gnu_cxx
       : _M_align(_S_align), _M_max_bytes(_S_max_bytes), _M_min_bin(_S_min_bin),
       _M_chunk_size(_S_chunk_size), _M_max_threads(_S_max_threads), 
       _M_freelist_headroom(_S_freelist_headroom), 
-      _M_force_new(getenv("GLIBCXX_FORCE_NEW") ? true : false)
+      _M_force_new(std::getenv("GLIBCXX_FORCE_NEW") ? true : false)
       { }
 
       explicit
@@ -149,7 +152,7 @@ namespace __gnu_cxx
     _M_get_binmap(size_t __bytes)
     { return _M_binmap[__bytes]; }
 
-    const size_t
+    size_t
     _M_get_align()
     { return _M_options._M_align; }
 
@@ -196,13 +199,13 @@ namespace __gnu_cxx
       union _Block_record
       {
 	// Points to the block_record of the next free block.
-	_Block_record* volatile         _M_next;
+	_Block_record* 			_M_next;
       };
 
       struct _Bin_record
       {
 	// An "array" of pointers to the first free block.
-	_Block_record** volatile        _M_first;
+	_Block_record**			_M_first;
 
 	// A list of the initial addresses of all allocated blocks.
 	_Block_address*		     	_M_address;
@@ -245,7 +248,7 @@ namespace __gnu_cxx
       // An "array" of bin_records each of which represents a specific
       // power of 2 size. Memory to this "array" is allocated in
       // _M_initialize().
-      _Bin_record* volatile	_M_bin;
+      _Bin_record*		 _M_bin;
       
       // Actual value calculated in _M_initialize().
       size_t 	       	     	_M_bin_size;     
@@ -272,7 +275,7 @@ namespace __gnu_cxx
       struct _Thread_record
       {
 	// Points to next free thread id record. NULL if last record in list.
-	_Thread_record* volatile        _M_next;
+	_Thread_record*			_M_next;
 	
 	// Thread id ranging from 1 to _S_max_threads.
 	size_t                          _M_id;
@@ -281,7 +284,7 @@ namespace __gnu_cxx
       union _Block_record
       {
 	// Points to the block_record of the next free block.
-	_Block_record* volatile         _M_next;
+	_Block_record*			_M_next;
 	
 	// The thread id of the thread which has requested this block.
 	size_t                          _M_thread_id;
@@ -292,17 +295,22 @@ namespace __gnu_cxx
 	// An "array" of pointers to the first free block for each
 	// thread id. Memory to this "array" is allocated in
 	// _S_initialize() for _S_max_threads + global pool 0.
-	_Block_record** volatile        _M_first;
+	_Block_record**			_M_first;
 	
 	// A list of the initial addresses of all allocated blocks.
 	_Block_address*		     	_M_address;
 
 	// An "array" of counters used to keep track of the amount of
 	// blocks that are on the freelist/used for each thread id.
-	// Memory to these "arrays" is allocated in _S_initialize() for
-	// _S_max_threads + global pool 0.
-	size_t* volatile                _M_free;
-	size_t* volatile                _M_used;
+	// - Note that the second part of the allocated _M_used "array"
+	//   actually hosts (atomic) counters of reclaimed blocks:  in
+	//   _M_reserve_block and in _M_reclaim_block those numbers are
+	//   subtracted from the first ones to obtain the actual size
+	//   of the "working set" of the given thread.
+	// - Memory to these "arrays" is allocated in _S_initialize()
+	//   for _S_max_threads + global pool 0.
+	size_t*				_M_free;
+	size_t*			        _M_used;
 	
 	// Each bin has its own mutex which is used to ensure data
 	// integrity while changing "ownership" on a block.  The mutex
@@ -366,7 +374,7 @@ namespace __gnu_cxx
       // An "array" of bin_records each of which represents a specific
       // power of 2 size. Memory to this "array" is allocated in
       // _M_initialize().
-      _Bin_record* volatile	_M_bin;
+      _Bin_record*		_M_bin;
 
       // Actual value calculated in _M_initialize().
       size_t 	       	     	_M_bin_size;
@@ -481,7 +489,7 @@ namespace __gnu_cxx
 			     sizeof(_Tp) * size_t(_Tune::_S_chunk_size),
 			     _Tune::_S_max_threads,
 			     _Tune::_S_freelist_headroom,
-			     getenv("GLIBCXX_FORCE_NEW") ? true : false);
+			     std::getenv("GLIBCXX_FORCE_NEW") ? true : false);
 	static pool_type _S_pool(_S_tune);
 	return _S_pool;
       }
@@ -586,7 +594,14 @@ namespace __gnu_cxx
       // 402. wrong new expression in [some_] allocator::construct
       void 
       construct(pointer __p, const _Tp& __val) 
-      { ::new(__p) _Tp(__val); }
+      { ::new((void *)__p) _Tp(__val); }
+
+#ifdef __GXX_EXPERIMENTAL_CXX0X__
+      template<typename... _Args>
+        void
+        construct(pointer __p, _Args&&... __args)
+	{ ::new((void *)__p) _Tp(std::forward<_Args>(__args)...); }
+#endif
 
       void 
       destroy(pointer __p) { __p->~_Tp(); }
@@ -733,6 +748,7 @@ namespace __gnu_cxx
     { return false; }
 
 #undef __thread_default
-} // namespace __gnu_cxx
+
+_GLIBCXX_END_NAMESPACE
 
 #endif
