@@ -13,9 +13,11 @@
 
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
+// { dg-require-fileio "" }
+
 // 27.8.1.3 filebuf member functions
 
 #include <locale>
@@ -62,10 +64,12 @@ void test01()
   locale loc(locale::classic(), cvt);
 
   filebuf fb;
+  fb.pubsetbuf(0, 0);
   fb.pubimbue(loc);
 
   fb.open(name, ios_base::out);
   fb.sputc('a');
+  fb.in_avail(); // showmanyc() should have no effect on close().
 
   VERIFY( !cvt->unshift_called );
   fb.close();
