@@ -1,8 +1,6 @@
 // { dg-require-namedlocale "" }
-// { dg-require-fork "" }
-// { dg-require-mkfifo "" }
 
-// 2004-01-11  Petur Runolfsson  <peturr02@ru.is>
+// 2004-04-16  Petur Runolfsson  <peturr02@ru.is>
 
 // Copyright (C) 2004, 2005 Free Software Foundation, Inc.
 //
@@ -26,54 +24,42 @@
 
 #include <fstream>
 #include <locale>
-
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-
 #include <testsuite_hooks.h>
 
-// libstdc++/13582
+class Buf : public std::filebuf
+{
+protected:
+  virtual int_type
+  overflow(int_type c = traits_type::eof())
+  {
+    return traits_type::eq_int_type(c, traits_type::eof()) ?
+      traits_type::eof() : std::filebuf::overflow(c);
+  }
+};
+
+// libstdc++/14975
 void test01()
 {
+  using namespace std;
   bool test __attribute__((unused)) = true;
-  using namespace std; 
-  using namespace __gnu_test;
 
-  locale loc_en(locale("en_US"));
-  locale loc_fr(locale("fr_FR"));
-
-  const char* name = "tmp_fifo_13582-2";
-  unlink(name);
-  mkfifo(name, S_IRWXU);
-
-  int child = fork();
-  if (child == 0)
+  Buf fb;
+  locale loc_us = locale("en_US");
+  fb.pubimbue(loc_us);
+  fb.open("tmp_14975-1", ios_base::out);
+  
+  try
+    {
+      fb.sputc('a');
+      fb.sputc('b');
+      fb.pubimbue(locale::classic());
+      fb.sputc('c');
+      fb.pubsync();
+      fb.close();
+    }
+  catch (std::exception&)
     {
-      filebuf fbout;
-      fbout.open(name, ios_base::out);
-      fbout.sputn("12345", 5);
-      fbout.pubsync();
-      fbout.close();
-      exit(0);
     }
-
-  filebuf fbin;
-  fbin.open(name, ios_base::in);
-  filebuf::int_type n = fbin.sbumpc();
-  VERIFY( n == '1' );
-  fbin.pubimbue(loc_en);
-  n = fbin.sbumpc();
-  VERIFY( n == '2' );
-  fbin.pubimbue(loc_fr);
-  n = fbin.sbumpc();
-  VERIFY( n == '3' );
-  n = fbin.sbumpc();
-  VERIFY( n == '4' );
-  n = fbin.sbumpc();
-  VERIFY( n == '5' );
-  n = fbin.sbumpc();
-  VERIFY( n == filebuf::traits_type::eof() );
 }
 
 int main()
