@@ -1,6 +1,6 @@
 // 2005-08-26  Paolo Carlini  <pcarlini@suse.de>
 //
-// Copyright (C) 2005 Free Software Foundation, Inc.
+// Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -32,13 +32,13 @@ test01()
   typedef std::tr1::array<int, len> array_type;
   
   {
-    array_type a = { 0, 1, 2, 3, 4 };
+    array_type a = { { 0, 1, 2, 3, 4 } };
     int* pi = a.data();
     VERIFY( *pi == 0 );
   }
 
   {
-    const array_type ca = { 4, 3, 2, 1, 0 };
+    const array_type ca = { { 4, 3, 2, 1, 0 } };
     const int* pci = ca.data();
     VERIFY( *pci == 4 );
   }
